﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using System;
using System.IO;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Data {
  [Item("PathValue", "Represents a path.")]
  [StorableClass]
  public abstract class PathValue : StringValue {
    [Storable]
    private bool checkExistence;
    public bool CheckExistence {
      get { return checkExistence; }
      set {
        if (ReadOnly) throw new NotSupportedException("CheckExistence cannot be set. PathValue is read-only.");
        if (value != checkExistence) {
          checkExistence = value;
          OnCheckExistenceChanged();
        }
      }
    }

    public override string Value {
      get { return base.Value; }
      set {
        if (value == null) value = string.Empty;
        if (checkExistence && !string.IsNullOrEmpty(value) && !File.Exists(value) && !Directory.Exists(value))
          throw new ArgumentException(string.Format("The given file {0} does not exists.", value));
        if (value[value.Length - 1] == Path.DirectorySeparatorChar)
          value = value.TrimEnd(Path.DirectorySeparatorChar);
        if (value[value.Length - 1] == Path.AltDirectorySeparatorChar)
          value = value.TrimEnd(Path.AltDirectorySeparatorChar);
        base.Value = value;
      }
    }

    [StorableConstructor]
    protected PathValue(bool deserializing) : base(deserializing) { }
    protected PathValue(PathValue original, Cloner cloner)
      : base(original, cloner) {
      checkExistence = original.checkExistence;
    }

    protected PathValue()
      : base(string.Empty) {
      checkExistence = true;
    }

    public abstract bool Exists();

    protected override bool Validate(string value, out string errorMessage) {
      if (CheckExistence && !File.Exists(value) && !Directory.Exists(value)) {
        errorMessage = string.Format("The given path {0} does not exist.", value);
        return false;
      }
      return base.Validate(value, out errorMessage);
    }

    public event EventHandler CheckExistenceChanged;
    protected virtual void OnCheckExistenceChanged() {
      var handler = CheckExistenceChanged;
      if (handler != null)
        handler(this, EventArgs.Empty);
    }
  }
}
